@@ -4,6 +4,9 @@ import (
 	"os"
 )
 
+// Configured at compile time
+var bashPath string
+
 func main() {
 	var env = GetEnv()
 	var args = os.Args
