@@ -7,8 +7,20 @@ const STDLIB = "#!bash\n" +
 	"set -e\n" +
 	"direnv=\"%s\"\n" +
 	"\n" +
+	"# Config, change in the direnvrc\n" +
 	"DIRENV_LOG_FORMAT=\"${DIRENV_LOG_FORMAT-direnv: %%s}\"\n" +
 	"\n" +
+	"# Usage: direnv_layout_dir\n" +
+	"#\n" +
+	"# Prints the folder path that direnv should use to store layout content.\n" +
+	"# This needs to be a function as $PWD might change during source_env/up.\n" +
+	"#\n" +
+	"# The output defaults to $PWD/.direnv.\n" +
+	"\n" +
+	"direnv_layout_dir() {\n" +
+	"  echo \"${direnv_layout_dir:-$PWD/.direnv}\"\n" +
+	"}\n" +
+	"\n" +
 	"# Usage: log_status [<message> ...]\n" +
 	"#\n" +
 	"# Logs a status message. Acts like echo,\n" +
@@ -43,7 +55,7 @@ const STDLIB = "#!bash\n" +
 	"  local color_normal\n" +
 	"  local color_error\n" +
 	"  color_normal=$(tput sgr0)\n" +
-	"  color_error=\"\\e[0;31m\"\n" +
+	"  color_error=$(tput setaf 1)\n" +
 	"  if [[ -n $DIRENV_LOG_FORMAT ]]; then\n" +
 	"    local msg=$*\n" +
 	"    # shellcheck disable=SC2059\n" +
@@ -145,7 +157,6 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"find_up() {\n" +
 	"  (\n" +
-	"    cd \"$(pwd -P 2>/dev/null)\"\n" +
 	"    while true; do\n" +
 	"      if [[ -f $1 ]]; then\n" +
 	"        echo \"$PWD/$1\"\n" +
@@ -174,7 +185,7 @@ const STDLIB = "#!bash\n" +
 	"  rcfile=$(user_rel_path \"$rcpath\")\n" +
 	"  watch_file \"$rcpath\"\n" +
 	"\n" +
-	"  pushd \"$(pwd -P 2>/dev/null)\" > /dev/null\n" +
+	"  pushd \"$(pwd 2>/dev/null)\" > /dev/null\n" +
 	"    pushd \"$(dirname \"$rcpath\")\" > /dev/null\n" +
 	"    if [[ -f ./$(basename \"$rcpath\") ]]; then\n" +
 	"      log_status \"loading $rcfile\"\n" +
@@ -195,7 +206,7 @@ const STDLIB = "#!bash\n" +
 	"watch_file() {\n" +
 	"  local file=${1/#\\~/$HOME}\n" +
 	"\n" +
-	"  eval \"$($direnv watch \"$file\")\"\n" +
+	"  eval \"$(\"$direnv\" watch \"$file\")\"\n" +
 	"}\n" +
 	"\n" +
 	"\n" +
@@ -212,7 +223,7 @@ const STDLIB = "#!bash\n" +
 	"  fi\n" +
 	"  dir=$(cd .. && find_up \"$file\")\n" +
 	"  if [[ -n $dir ]]; then\n" +
-	"    source_env \"$(user_rel_path \"$dir\")\"\n" +
+	"    source_env \"$dir\"\n" +
 	"  fi\n" +
 	"}\n" +
 	"\n" +
@@ -234,39 +245,48 @@ const STDLIB = "#!bash\n" +
 	"  eval \"$exports\"\n" +
 	"}\n" +
 	"\n" +
-	"# Usage: PATH_add <path>\n" +
+	"# Usage: PATH_add <path> [<path> ...]\n" +
 	"#\n" +
-	"# Prepends the expanded <path> to the PATH environment variable. It prevents a\n" +
-	"# common mistake where PATH is replaced by only the new <path>.\n" +
+	"# Prepends the expanded <path> to the PATH environment variable, in order.\n" +
+	"# It prevents a common mistake where PATH is replaced by only the new <path>,\n" +
+	"# or where a trailing colon is left in PATH, resulting in the current directory\n" +
+	"# being considered in the PATH.  Supports adding multiple directories at once.\n" +
 	"#\n" +
 	"# Example:\n" +
 	"#\n" +
 	"#    pwd\n" +
-	"#    # output: /home/user/my/project\n" +
+	"#    # output: /my/project\n" +
 	"#    PATH_add bin\n" +
 	"#    echo $PATH\n" +
-	"#    # output: /home/user/my/project/bin:/usr/bin:/bin\n" +
+	"#    # output: /my/project/bin:/usr/bin:/bin\n" +
+	"#    PATH_add bam boum\n" +
+	"#    echo $PATH\n" +
+	"#    # output: /my/project/bam:/my/project/boum:/my/project/bin:/usr/bin:/bin\n" +
 	"#\n" +
 	"PATH_add() {\n" +
-	"  PATH=$(expand_path \"$1\"):$PATH\n" +
-	"  export PATH\n" +
+	"  path_add PATH \"$@\"\n" +
 	"}\n" +
 	"\n" +
-	"# Usage: path_add <varname> <path>\n" +
+	"# Usage: path_add <varname> <path> [<path> ...]\n" +
 	"#\n" +
 	"# Works like PATH_add except that it's for an arbitrary <varname>.\n" +
 	"path_add() {\n" +
-	"  local old_paths=\"${!1}\"\n" +
-	"  local dir\n" +
-	"  dir=$(expand_path \"$2\")\n" +
+	"  local var_name=\"$1\"\n" +
+	"  # split existing paths into an array\n" +
+	"  declare -a path_array\n" +
+	"  IFS=: read -ra path_array <<< \"${!1}\"\n" +
+	"  shift\n" +
 	"\n" +
-	"  if [[ -z $old_paths ]]; then\n" +
-	"    old_paths=\"$dir\"\n" +
-	"  else\n" +
-	"    old_paths=\"$dir:$old_paths\"\n" +
-	"  fi\n" +
+	"  # prepend the passed paths in the right order\n" +
+	"  for (( i=$# ; i>0 ; i-- )); do\n" +
+	"    path_array=( \"$(expand_path \"${!i}\")\" \"${path_array[@]}\" )\n" +
+	"  done\n" +
 	"\n" +
-	"  export \"$1=$old_paths\"\n" +
+	"  # join back all the paths\n" +
+	"  local path=$(IFS=:; echo \"${path_array[*]}\")\n" +
+	"\n" +
+	"  # and finally export back the result to the original variable\n" +
+	"  export \"$var_name=$path\"\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: MANPATH_add <path>\n" +
@@ -352,7 +372,7 @@ const STDLIB = "#!bash\n" +
 	"# See http://search.cpan.org/dist/local-lib/lib/local/lib.pm for more details\n" +
 	"#\n" +
 	"layout_perl() {\n" +
-	"  local libdir=$PWD/.direnv/perl5\n" +
+	"  local libdir=$(direnv_layout_dir)/perl5\n" +
 	"  export LOCAL_LIB_DIR=$libdir\n" +
 	"  export PERL_MB_OPT=\"--install_base '$libdir'\"\n" +
 	"  export PERL_MM_OPT=\"INSTALL_BASE=$libdir\"\n" +
@@ -364,7 +384,7 @@ const STDLIB = "#!bash\n" +
 	"# Usage: layout python <python_exe>\n" +
 	"#\n" +
 	"# Creates and loads a virtualenv environment under\n" +
-	"# \"$PWD/.direnv/python-$python_version\".\n" +
+	"# \"$direnv_layout_dir/python-$python_version\".\n" +
 	"# This forces the installation of any egg into the project's sub-folder.\n" +
 	"#\n" +
 	"# It's possible to specify the python executable if you want to use different\n" +
@@ -372,7 +392,8 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"layout_python() {\n" +
 	"  local python=${1:-python}\n" +
-	"  local old_env=$PWD/.direnv/virtualenv\n" +
+	"  [[ $# -gt 0 ]] && shift\n" +
+	"  local old_env=$(direnv_layout_dir)/virtualenv\n" +
 	"  unset PYTHONHOME\n" +
 	"  if [[ -d $old_env && $python = python ]]; then\n" +
 	"    export VIRTUAL_ENV=$old_env\n" +
@@ -384,38 +405,97 @@ const STDLIB = "#!bash\n" +
 	"      return 1\n" +
 	"    fi\n" +
 	"\n" +
-	"    export VIRTUAL_ENV=$PWD/.direnv/python-$python_version\n" +
+	"    export VIRTUAL_ENV=$(direnv_layout_dir)/python-$python_version\n" +
 	"    if [[ ! -d $VIRTUAL_ENV ]]; then\n" +
-	"      virtualenv \"--python=$python\" \"$VIRTUAL_ENV\"\n" +
+	"      virtualenv \"--python=$python\" \"$@\" \"$VIRTUAL_ENV\"\n" +
 	"    fi\n" +
 	"  fi\n" +
 	"  PATH_add \"$VIRTUAL_ENV/bin\"\n" +
 	"}\n" +
 	"\n" +
+	"# Usage: layout python2\n" +
+	"#\n" +
+	"# A shortcut for $(layout python python2)\n" +
+	"#\n" +
+	"layout_python2() {\n" +
+	"  layout_python python2 \"$@\"\n" +
+	"}\n" +
+	"\n" +
 	"# Usage: layout python3\n" +
 	"#\n" +
 	"# A shortcut for $(layout python python3)\n" +
 	"#\n" +
 	"layout_python3() {\n" +
-	"  layout_python python3\n" +
+	"  layout_python python3 \"$@\"\n" +
+	"}\n" +
+	"\n" +
+	"# Usage: layout anaconda <enviroment_name> [<conda_exe>]\n" +
+	"#\n" +
+	"# Activates anaconda for the named environment. If the environment\n" +
+	"# hasn't been created, it will be using the environment.yml file in\n" +
+	"# the current directory. <conda_exe> is optional and will default to\n" +
+	"# the one found in the system environment.\n" +
+	"#\n" +
+	"layout_anaconda() {\n" +
+	"  local env_name=$1\n" +
+	"  local conda\n" +
+	"  if [[ $# -gt 1 ]]; then\n" +
+	"    conda=${2}\n" +
+	"  else\n" +
+	"    conda=$(command -v conda)\n" +
+	"  fi\n" +
+	"  PATH_add $(dirname \"$conda\")\n" +
+	"  local env_loc=$(\"$conda\" env list | grep -- \"$env_name\")\n" +
+	"  if [[ ! \"$env_loc\" == $env_name*$env_name ]]; then\n" +
+	"    if [[ -e environment.yml ]]; then\n" +
+	"      log_status \"creating conda enviroment\"\n" +
+	"      \"$conda\" env create\n" +
+	"    else\n" +
+	"      log_error \"Could not find environment.yml\"\n" +
+	"      return 1\n" +
+	"    fi\n" +
+	"  fi\n" +
+	"\n" +
+	"  source activate \"$env_name\"\n" +
+	"}\n" +
+	"\n" +
+	"# Usage: layout pipenv\n" +
+	"#\n" +
+	"# Similar to layout_python, but uses Pipenv to build a\n" +
+	"# virtualenv from the Pipfile located in the same directory.\n" +
+	"#\n" +
+	"layout_pipenv() {\n" +
+	"  if [[ ! -f Pipfile ]]; then\n" +
+	"    log_error 'No Pipfile found.  Use `pipenv` to create a Pipfile first.'\n" +
+	"    exit 2\n" +
+	"  fi\n" +
+	"\n" +
+	"  local VENV=$(pipenv --bare --venv 2>/dev/null)\n" +
+	"  if [[ -z $VENV || ! -d $VENV ]]; then\n" +
+	"    pipenv install --dev\n" +
+	"  fi\n" +
+	"\n" +
+	"  export VIRTUAL_ENV=$(pipenv --venv)\n" +
+	"  PATH_add \"$VIRTUAL_ENV/bin\"\n" +
+	"  export PIPENV_ACTIVE=1\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: layout ruby\n" +
 	"#\n" +
-	"# Sets the GEM_HOME environment variable to \"$PWD/.direnv/ruby/RUBY_VERSION\".\n" +
+	"# Sets the GEM_HOME environment variable to \"$(direnv_layout_dir)/ruby/RUBY_VERSION\".\n" +
 	"# This forces the installation of any gems into the project's sub-folder.\n" +
 	"# If you're using bundler it will create wrapper programs that can be invoked\n" +
 	"# directly instead of using the $(bundle exec) prefix.\n" +
 	"#\n" +
 	"layout_ruby() {\n" +
 	"  if ruby -e \"exit Gem::VERSION > '2.2.0'\" 2>/dev/null; then\n" +
-	"    export GEM_HOME=$PWD/.direnv/ruby\n" +
+	"    export GEM_HOME=$(direnv_layout_dir)/ruby\n" +
 	"  else\n" +
 	"    local ruby_version\n" +
 	"    ruby_version=$(ruby -e\"puts (defined?(RUBY_ENGINE) ? RUBY_ENGINE : 'ruby') + '-' + RUBY_VERSION\")\n" +
-	"    export GEM_HOME=$PWD/.direnv/ruby-${ruby_version}\n" +
+	"    export GEM_HOME=$(direnv_layout_dir)/ruby-${ruby_version}\n" +
 	"  fi\n" +
-	"  export BUNDLE_BIN=$PWD/.direnv/bin\n" +
+	"  export BUNDLE_BIN=$(direnv_layout_dir)/bin\n" +
 	"\n" +
 	"  PATH_add \"$GEM_HOME/bin\"\n" +
 	"  PATH_add \"$BUNDLE_BIN\"\n" +
@@ -488,7 +568,7 @@ const STDLIB = "#!bash\n" +
 	"use_node() {\n" +
 	"  local version=$1\n" +
 	"  local via=\"\"\n" +
-	"  local node_version_prefix=${NODE_VERSION_PREFIX:-node-v}\n" +
+	"  local node_version_prefix=${NODE_VERSION_PREFIX-node-v}\n" +
 	"  local node_wanted\n" +
 	"  local node_prefix\n" +
 	"\n" +
@@ -557,11 +637,20 @@ const STDLIB = "#!bash\n" +
 	"# (e.g `use nix -p ocaml`).\n" +
 	"#\n" +
 	"use_nix() {\n" +
+	"  local orig_IN_NIX_SHELL=\"$IN_NIX_SHELL\"\n" +
+	"\n" +
 	"  direnv_load nix-shell --show-trace \"$@\" --run 'direnv dump'\n" +
 	"  if [[ $# = 0 ]]; then\n" +
 	"    watch_file default.nix\n" +
 	"    watch_file shell.nix\n" +
 	"  fi\n" +
+	"\n" +
+	"  # Don't change the IN_NIX_SHELL env var\n" +
+	"  if [[ -z $orig_IN_NIX_SHELL ]]; then\n" +
+	"    unset IN_NIX_SHELL\n" +
+	"  else\n" +
+	"    export IN_NIX_SHELL=\"$orig_IN_NIX_SHELL\"\n" +
+	"  fi\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: use_guix [...]\n" +
@@ -580,7 +669,8 @@ const STDLIB = "#!bash\n" +
 	"\n" +
 	"## Load the global ~/.direnvrc if present\n" +
 	"if [[ -f ${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc ]]; then\n" +
-	"  source_env \"${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc\" >&2\n" +
+	"  source \"${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc\" >&2\n" +
 	"elif [[ -f $HOME/.direnvrc ]]; then\n" +
-	"  source_env \"$HOME/.direnvrc\" >&2\n" +
-	"fi\n"
+	"  source \"$HOME/.direnvrc\" >&2\n" +
+	"fi\n" +
+	""
