@@ -5,8 +5,20 @@
 set -e
 direnv="%s"
 
+# Config, change in the direnvrc
 DIRENV_LOG_FORMAT="${DIRENV_LOG_FORMAT-direnv: %%s}"
 
+# Usage: direnv_layout_dir
+#
+# Prints the folder path that direnv should use to store layout content.
+# This needs to be a function as $PWD might change during source_env/up.
+#
+# The output defaults to $PWD/.direnv.
+
+direnv_layout_dir() {
+  echo "${direnv_layout_dir:-$PWD/.direnv}"
+}
+
 # Usage: log_status [<message> ...]
 #
 # Logs a status message. Acts like echo,
@@ -41,7 +53,7 @@ log_error() {
   local color_normal
   local color_error
   color_normal=$(tput sgr0)
-  color_error="\e[0;31m"
+  color_error=$(tput setaf 1)
   if [[ -n $DIRENV_LOG_FORMAT ]]; then
     local msg=$*
     # shellcheck disable=SC2059
@@ -143,7 +155,6 @@ user_rel_path() {
 #
 find_up() {
   (
-    cd "$(pwd -P 2>/dev/null)"
     while true; do
       if [[ -f $1 ]]; then
         echo "$PWD/$1"
@@ -172,7 +183,7 @@ source_env() {
   rcfile=$(user_rel_path "$rcpath")
   watch_file "$rcpath"
 
-  pushd "$(pwd -P 2>/dev/null)" > /dev/null
+  pushd "$(pwd 2>/dev/null)" > /dev/null
     pushd "$(dirname "$rcpath")" > /dev/null
     if [[ -f ./$(basename "$rcpath") ]]; then
       log_status "loading $rcfile"
@@ -193,7 +204,7 @@ source_env() {
 watch_file() {
   local file=${1/#\~/$HOME}
 
-  eval "$($direnv watch "$file")"
+  eval "$("$direnv" watch "$file")"
 }
 
 
@@ -210,7 +221,7 @@ source_up() {
   fi
   dir=$(cd .. && find_up "$file")
   if [[ -n $dir ]]; then
-    source_env "$(user_rel_path "$dir")"
+    source_env "$dir"
   fi
 }
 
@@ -232,39 +243,48 @@ direnv_load() {
   eval "$exports"
 }
 
-# Usage: PATH_add <path>
+# Usage: PATH_add <path> [<path> ...]
 #
-# Prepends the expanded <path> to the PATH environment variable. It prevents a
-# common mistake where PATH is replaced by only the new <path>.
+# Prepends the expanded <path> to the PATH environment variable, in order.
+# It prevents a common mistake where PATH is replaced by only the new <path>,
+# or where a trailing colon is left in PATH, resulting in the current directory
+# being considered in the PATH.  Supports adding multiple directories at once.
 #
 # Example:
 #
 #    pwd
-#    # output: /home/user/my/project
+#    # output: /my/project
 #    PATH_add bin
 #    echo $PATH
-#    # output: /home/user/my/project/bin:/usr/bin:/bin
+#    # output: /my/project/bin:/usr/bin:/bin
+#    PATH_add bam boum
+#    echo $PATH
+#    # output: /my/project/bam:/my/project/boum:/my/project/bin:/usr/bin:/bin
 #
 PATH_add() {
-  PATH=$(expand_path "$1"):$PATH
-  export PATH
+  path_add PATH "$@"
 }
 
-# Usage: path_add <varname> <path>
+# Usage: path_add <varname> <path> [<path> ...]
 #
 # Works like PATH_add except that it's for an arbitrary <varname>.
 path_add() {
-  local old_paths="${!1}"
-  local dir
-  dir=$(expand_path "$2")
+  local var_name="$1"
+  # split existing paths into an array
+  declare -a path_array
+  IFS=: read -ra path_array <<< "${!1}"
+  shift
 
-  if [[ -z $old_paths ]]; then
-    old_paths="$dir"
-  else
-    old_paths="$dir:$old_paths"
-  fi
+  # prepend the passed paths in the right order
+  for (( i=$# ; i>0 ; i-- )); do
+    path_array=( "$(expand_path "${!i}")" "${path_array[@]}" )
+  done
 
-  export "$1=$old_paths"
+  # join back all the paths
+  local path=$(IFS=:; echo "${path_array[*]}")
+
+  # and finally export back the result to the original variable
+  export "$var_name=$path"
 }
 
 # Usage: MANPATH_add <path>
@@ -350,7 +370,7 @@ layout_node() {
 # See http://search.cpan.org/dist/local-lib/lib/local/lib.pm for more details
 #
 layout_perl() {
-  local libdir=$PWD/.direnv/perl5
+  local libdir=$(direnv_layout_dir)/perl5
   export LOCAL_LIB_DIR=$libdir
   export PERL_MB_OPT="--install_base '$libdir'"
   export PERL_MM_OPT="INSTALL_BASE=$libdir"
@@ -362,7 +382,7 @@ layout_perl() {
 # Usage: layout python <python_exe>
 #
 # Creates and loads a virtualenv environment under
-# "$PWD/.direnv/python-$python_version".
+# "$direnv_layout_dir/python-$python_version".
 # This forces the installation of any egg into the project's sub-folder.
 #
 # It's possible to specify the python executable if you want to use different
@@ -370,7 +390,8 @@ layout_perl() {
 #
 layout_python() {
   local python=${1:-python}
-  local old_env=$PWD/.direnv/virtualenv
+  [[ $# -gt 0 ]] && shift
+  local old_env=$(direnv_layout_dir)/virtualenv
   unset PYTHONHOME
   if [[ -d $old_env && $python = python ]]; then
     export VIRTUAL_ENV=$old_env
@@ -382,38 +403,97 @@ layout_python() {
       return 1
     fi
 
-    export VIRTUAL_ENV=$PWD/.direnv/python-$python_version
+    export VIRTUAL_ENV=$(direnv_layout_dir)/python-$python_version
     if [[ ! -d $VIRTUAL_ENV ]]; then
-      virtualenv "--python=$python" "$VIRTUAL_ENV"
+      virtualenv "--python=$python" "$@" "$VIRTUAL_ENV"
     fi
   fi
   PATH_add "$VIRTUAL_ENV/bin"
 }
 
+# Usage: layout python2
+#
+# A shortcut for $(layout python python2)
+#
+layout_python2() {
+  layout_python python2 "$@"
+}
+
 # Usage: layout python3
 #
 # A shortcut for $(layout python python3)
 #
 layout_python3() {
-  layout_python python3
+  layout_python python3 "$@"
+}
+
+# Usage: layout anaconda <enviroment_name> [<conda_exe>]
+#
+# Activates anaconda for the named environment. If the environment
+# hasn't been created, it will be using the environment.yml file in
+# the current directory. <conda_exe> is optional and will default to
+# the one found in the system environment.
+#
+layout_anaconda() {
+  local env_name=$1
+  local conda
+  if [[ $# -gt 1 ]]; then
+    conda=${2}
+  else
+    conda=$(command -v conda)
+  fi
+  PATH_add $(dirname "$conda")
+  local env_loc=$("$conda" env list | grep -- "$env_name")
+  if [[ ! "$env_loc" == $env_name*$env_name ]]; then
+    if [[ -e environment.yml ]]; then
+      log_status "creating conda enviroment"
+      "$conda" env create
+    else
+      log_error "Could not find environment.yml"
+      return 1
+    fi
+  fi
+
+  source activate "$env_name"
+}
+
+# Usage: layout pipenv
+#
+# Similar to layout_python, but uses Pipenv to build a
+# virtualenv from the Pipfile located in the same directory.
+#
+layout_pipenv() {
+  if [[ ! -f Pipfile ]]; then
+    log_error 'No Pipfile found.  Use `pipenv` to create a Pipfile first.'
+    exit 2
+  fi
+
+  local VENV=$(pipenv --bare --venv 2>/dev/null)
+  if [[ -z $VENV || ! -d $VENV ]]; then
+    pipenv install --dev
+  fi
+
+  export VIRTUAL_ENV=$(pipenv --venv)
+  PATH_add "$VIRTUAL_ENV/bin"
+  export PIPENV_ACTIVE=1
 }
 
 # Usage: layout ruby
 #
-# Sets the GEM_HOME environment variable to "$PWD/.direnv/ruby/RUBY_VERSION".
+# Sets the GEM_HOME environment variable to "$(direnv_layout_dir)/ruby/RUBY_VERSION".
 # This forces the installation of any gems into the project's sub-folder.
 # If you're using bundler it will create wrapper programs that can be invoked
 # directly instead of using the $(bundle exec) prefix.
 #
 layout_ruby() {
   if ruby -e "exit Gem::VERSION > '2.2.0'" 2>/dev/null; then
-    export GEM_HOME=$PWD/.direnv/ruby
+    export GEM_HOME=$(direnv_layout_dir)/ruby
   else
     local ruby_version
     ruby_version=$(ruby -e"puts (defined?(RUBY_ENGINE) ? RUBY_ENGINE : 'ruby') + '-' + RUBY_VERSION")
-    export GEM_HOME=$PWD/.direnv/ruby-${ruby_version}
+    export GEM_HOME=$(direnv_layout_dir)/ruby-${ruby_version}
   fi
-  export BUNDLE_BIN=$PWD/.direnv/bin
+  export BUNDLE_BIN=$(direnv_layout_dir)/bin
 
   PATH_add "$GEM_HOME/bin"
   PATH_add "$BUNDLE_BIN"
@@ -486,7 +566,7 @@ rvm() {
 use_node() {
   local version=$1
   local via=""
-  local node_version_prefix=${NODE_VERSION_PREFIX:-node-v}
+  local node_version_prefix=${NODE_VERSION_PREFIX-node-v}
   local node_wanted
   local node_prefix
 
@@ -555,11 +635,20 @@ use_node() {
 # (e.g `use nix -p ocaml`).
 #
 use_nix() {
+  local orig_IN_NIX_SHELL="$IN_NIX_SHELL"
+
   direnv_load nix-shell --show-trace "$@" --run 'direnv dump'
   if [[ $# = 0 ]]; then
     watch_file default.nix
     watch_file shell.nix
   fi
+
+  # Don't change the IN_NIX_SHELL env var
+  if [[ -z $orig_IN_NIX_SHELL ]]; then
+    unset IN_NIX_SHELL
+  else
+    export IN_NIX_SHELL="$orig_IN_NIX_SHELL"
+  fi
 }
 
 # Usage: use_guix [...]
@@ -578,7 +667,7 @@ use_guix() {
 
 ## Load the global ~/.direnvrc if present
 if [[ -f ${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc ]]; then
-  source_env "${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc" >&2
+  source "${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc" >&2
 elif [[ -f $HOME/.direnvrc ]]; then
-  source_env "$HOME/.direnvrc" >&2
+  source "$HOME/.direnvrc" >&2
 fi
